@@ -1,4 +1,4 @@
-use crate::NumberTheory;
+use crate::traits::NumberTheory;
 
 use crate::arithmetic::sign::Sign;
 use crate::arithmetic::mpz::Mpz;
@@ -10,18 +10,32 @@ impl NumberTheory for Mpz{
   
   
   fn quadratic_residue(&self, n: &Self) -> Self{
-      let mut p = self.clone();
+  
+     if n == &Mpz::zero(){
+        return Mpz::zero()
+     }
+     
+   let mut p = self.clone();
+   
     if p.sign == Sign::Negative {
-      p = p.add_modinv(&n);
+       p = p.add_modinv(&n);
     }
-     p.u_quadratic_residue(n)
+    
+     self.ref_product(&self).euclidean(n).1
   }
   
+  
   fn mul_mod(&self, other: &Self, n: &Self) -> Self{
-        let mut p = self.clone();
-        let mut q = other.clone();
+       
+       if n == &Mpz::zero(){
+        return Mpz::zero()
+       }
+       
+      let mut p = self.clone();
+      let mut q = other.clone();
+      
     if p.sign == Sign::Negative {
-      p = p.add_modinv(n);
+        p = p.add_modinv(n);
     }
     
     if q.sign == Sign::Negative{
@@ -29,15 +43,19 @@ impl NumberTheory for Mpz{
     }
   
       p.ref_product(&q).euclidean(n).1
-      
-      
-  }
+    }
   
   fn mod_pow(&self,  y: &Self, n: &Self )->Self{
-        let mut p = self.clone();
-    if p.sign == Sign::Negative {
-      p = p.add_modinv(&n);
-    }
+       if n == &Mpz::zero(){
+         return Mpz::zero()
+       }
+       
+     let mut p = self.clone();
+     
+       if p.sign == Sign::Negative {
+          p = p.add_modinv(&n);
+       }
+       
      p.u_mod_pow(y,n)
     }
     
@@ -56,10 +74,7 @@ impl NumberTheory for Mpz{
      }
    }
    
-   for i in 0..5{
-     let z = rand();
-     if self.sprp(z)==false{ return false} //
-   }
+   if self.sprp_check(5)==false{return false}
    
    return true
  
@@ -84,12 +99,12 @@ impl NumberTheory for Mpz{
     
 
     
-    
+    //115792089237316195423570985008687907853269984665640564039457584007913129639935
   fn factor(&self) -> Vec<Self>{
      let mut n = self.clone();
      let mut factors : Vec<Self> = vec![];
      let twofactor = n.trailing_zeros();
-
+     // println!("twos {}", twofactor);
      if twofactor > 0{
       n.mut_shr(twofactor as usize);
      factors.push(Mpz::from_u64(2u64));
@@ -101,6 +116,7 @@ impl NumberTheory for Mpz{
         
          
           if rem == 0{
+        //  println!("rem hit {}", i);
            let mut count = 1u64;
              factors.push(Mpz::from_u64(*i as u64));
              n = quo;
@@ -122,29 +138,35 @@ impl NumberTheory for Mpz{
 
           
           }
-
+          n.normalize();
+       // println!("{:?}", n);
+          if n == Mpz::one() {
+            return factors
+          }
           
            if n.sprp_check(5){   // stops if prime 
              factors.push(n.clone());
              factors.push(Mpz::one());
              return factors
           }
-        //  println!("Next step");
+         // println!("next step{}", n.to_string());
        'outer : while n != Mpz::one(){
           
               let k = n.rho_mpz();
-            //  println!("first rho");
+            //  println!("factor found {}", k.to_string());
               factors.push(k.clone());
               let mut count = 0u64;
-             
+            // println!("{:?}", n);
      'inner : loop {
             let (inner_quo, inner_rem) = n.euclidean(&k);
-             
+                          //println!("{:?}", n);
              if inner_rem != Mpz::zero(){
                 break 'inner;
              }
              n = inner_quo;
+             n.normalize(); // remove ? 
              count+=1;
+        //     println!("{:?}", n.to_string());
            }
            factors.push(Mpz::from_u64(count));
 
@@ -159,7 +181,9 @@ impl NumberTheory for Mpz{
   }
   
   fn radical(&self) -> Self{
+  
     let mut rad = Mpz::one();
+    
     for i in self.factor().iter().step_by(2){
       rad = rad.ref_product(i)
     }
