@@ -100,15 +100,17 @@ impl NumberTheory for u8{
  }
  
  fn quadratic_residue(&self, n: &Self) -> Self{
+      if n == &0 {return 0u8}
      ((*self as u16 * *self as u16) % *n as u16) as u8
  }
  
  fn mul_mod(&self, other: &Self, n: &Self) -> Self{
+           if n == &0 {return 0u8}
     ((*self as u16 * *other as u16) % *n as u16) as u8
  }
  
  fn mod_pow(&self, p: &Self, modulus: &Self)-> u8{  
-
+      if modulus == &0 {return 0u8}
   let mut z = 1u16;
   let mut base = *self as u16;
   let n = modulus.clone() as u16;
