@@ -8,12 +8,14 @@ use crate::fjprime64::fjprime_64;
 impl NumberTheory for u64{
 
  fn is_prime(&self)->bool{
-   if *self == 1 || *self == 0 {
-     return false
-   }
+   
    if *self < u32::MAX as u64 { // tree down to u32 if it fits
      return (*self as u32).is_prime()
    }
+   for i in PRIMELIST[..30].iter(){
+     if *self%*i as u64 == 0 {return false}
+   }
+   
      fjprime_64(*self)  
    
  }
@@ -122,15 +124,17 @@ impl NumberTheory for u64{
  }
  
  fn quadratic_residue(&self, n: &Self) -> Self{
+                if n == &0 {return 0u64}
      ((*self as u128 * *self as u128) % *n as u128) as u64
  }
  
  fn mul_mod(&self, other: &Self, n: &Self) -> Self{
+                if n == &0 {return 0u64}
     ((*self as u128 * *other as u128) % *n as u128) as u64
  }
  
  fn mod_pow(&self, p: &Self, modulus: &Self)-> Self{  
-
+                  if modulus == &0 {return 0u64}
   let mut z = 1u128;
   let mut base = *self as u128;
   let n = modulus.clone() as u128;
