@@ -8,12 +8,14 @@ use crate::fjprime32::fjprime_32;
 impl NumberTheory for u32{
 
  fn is_prime(&self)->bool{
-   if *self == 1 || *self == 0 {
-     return false
-   }
+  
    if *self < u16::MAX as u32 { // tree down to u16 if it fits
      return (*self as u16).is_prime()
    }
+   
+   for i in PRIMELIST[..30].iter(){
+     if *self%*i as u32 == 0{return false}
+   }
      fjprime_32(*self)  
    
  }
@@ -119,15 +121,17 @@ impl NumberTheory for u32{
  }
  
  fn quadratic_residue(&self, n: &Self) -> Self{
+           if n == &0 {return 0u32}
      ((*self as u64 * *self as u64) % *n as u64) as u32
  }
  
  fn mul_mod(&self, other: &Self, n: &Self) -> Self{
+                if n == &0 {return 0u32}
     ((*self as u64 * *other as u64) % *n as u64) as u32
  }
  
  fn mod_pow(&self, p: &Self, modulus: &Self)-> Self{  
-
+                if modulus == &0 {return 0u32}
   let mut z = 1u64;
   let mut base = *self as u64;
   let n = modulus.clone() as u64;
