@@ -13,9 +13,7 @@ pub const PRIMELIST: [u16; 54] = [// list of all primes less than 2^8
 impl NumberTheory for u16{
 
  fn is_prime(&self)->bool{
-   if *self == 1 || *self == 0 {
-     return false
-   }
+   
    if *self < u8::MAX as u16 { // tree down to u8 if it fits
      return (*self as u8).is_prime()
    }
@@ -113,15 +111,17 @@ impl NumberTheory for u16{
  }
  
  fn quadratic_residue(&self, n: &Self) -> Self{
+           if n == &0 {return 0u16}
      ((*self as u32 * *self as u32) % *n as u32) as u16
  }
  
  fn mul_mod(&self, other: &Self, n: &Self) -> Self{
+          if n == &0 {return 0u16}
     ((*self as u32 * *other as u32) % *n as u32) as u16
  }
  
  fn mod_pow(&self, p: &Self, modulus: &Self)-> Self{  
-
+            if modulus == &0 {return 0u16}
   let mut z = 1u32;
   let mut base = *self as u32;
   let n = modulus.clone() as u32;
